@@ -949,7 +949,7 @@ bool TorrentImpl::hasMissingFiles() const
 
 bool TorrentImpl::hasError() const
 {
-    return static_cast<bool>(m_nativeStatus.errc);
+    return (m_nativeStatus.errc || (m_nativeStatus.flags & lt::torrent_flags::upload_mode));
 }
 
 bool TorrentImpl::hasFilteredPieces() const
@@ -968,7 +968,19 @@ int TorrentImpl::queuePosition() const
 
 QString TorrentImpl::error() const
 {
-    return QString::fromStdString(m_nativeStatus.errc.message());
+    if (m_nativeStatus.errc)
+        return QString::fromStdString(m_nativeStatus.errc.message());
+
+    if (m_nativeStatus.flags & lt::torrent_flags::upload_mode)
+    {
+        const QString writeErrorStr = tr("Couldn't write to file.");
+        const QString uploadModeStr = tr("Torrent is currently in \"upload only\" mode.");
+        const QString errorMessage = QString::fromLocal8Bit(m_lastFileError.error.message().c_str());
+
+        return writeErrorStr + QLatin1Char(' ') + errorMessage + QLatin1String(". ") + uploadModeStr;
+    }
+
+    return {};
 }
 
 qlonglong TorrentImpl::totalDownload() const
@@ -1527,7 +1539,10 @@ void TorrentImpl::pause()
 void TorrentImpl::resume(const TorrentOperatingMode mode)
 {
     if (hasError())
+    {
         m_nativeHandle.clear_error();
+        m_nativeHandle.unset_flags(lt::torrent_flags::upload_mode);
+    }
 
     m_operatingMode = mode;
 
@@ -1750,6 +1765,9 @@ void TorrentImpl::handleSaveResumeDataAlert(const lt::save_resume_data_alert *p)
 
     m_ltAddTorrentParams.added_time = addedTime().toSecsSinceEpoch();
 
+    // We shouldn't save upload_mode flag to allow torrent operate normally on next run
+    m_ltAddTorrentParams.flags &= ~lt::torrent_flags::upload_mode;
+
     if (m_maintenanceJob == MaintenanceJob::HandleMetadata)
     {
         m_ltAddTorrentParams.have_pieces.clear();
@@ -1877,6 +1895,11 @@ void TorrentImpl::handleFileCompletedAlert(const lt::file_completed_alert *p)
     }
 }
 
+void TorrentImpl::handleFileErrorAlert(const lt::file_error_alert *p)
+{
+    m_lastFileError = {p->error, p->op};
+}
+
 void TorrentImpl::handleMetadataReceivedAlert(const lt::metadata_received_alert *p)
 {
     Q_UNUSED(p);
@@ -1923,6 +1946,9 @@ void TorrentImpl::handleAlert(const lt::alert *a)
     case lt::file_completed_alert::alert_type:
         handleFileCompletedAlert(static_cast<const lt::file_completed_alert*>(a));
         break;
+    case lt::file_error_alert::alert_type:
+        handleFileErrorAlert(static_cast<const lt::file_error_alert*>(a));
+        break;
     case lt::torrent_finished_alert::alert_type:
         handleTorrentFinishedAlert(static_cast<const lt::torrent_finished_alert*>(a));
         break;
